;**************************************************************************************************
;*                                                                                                *
;*    RAM Test for the C02 Pocket SBC series - you just never know when your memory is going!     *
;*                                                                                                *
;*              Equates and variables used in the C02BIOS and Monitor programs                    *
;*                                                                                                *
;*                  Updated for BIOS and Monitor versions starting at 4.00                        *
;*                                                                                                *
;*      This base of code should be at the start of any user created code. It provides            *
;*      equates to all known Page zero locations, subroutines and variables used.                 *
;*      This greatly simplifies access to any required resources in the existing code.            *
;*                                                                                                *
;**************************************************************************************************
        PL      66      ;Page Length
        PW      132     ;Page Width (# of char/line)
        CHIP    W65C02S ;Enable WDC 65C02 instructions
        PASS1   OFF     ;Set ON when used for debug
        INCLIST ON      ;Set ON for listing Include files
;**************************************************************************************************
;                                                                                                 *
;       - Page Zero definitions ($00 to $9F reserved for user routines)                           *
;                                                                                                 *
;**************************************************************************************************
;
        NOLIST                                  ;Turn listing off for this stuff
;
PGZERO_ST       .EQU    $A0                     ;Start of Monitor Page 0 use ($A0-$CF, 48 bytes)
;
BUFF_PG0        .EQU    PGZERO_ST+00            ;Default Page zero location for Monitor buffers
;
INBUFF          .EQU    BUFF_PG0+00             ;Input Buffer - 4 bytes ($A0-$A3)
DATABUFF        .EQU    BUFF_PG0+04             ;Data Buffer - 6 bytes ($A4-$A9)
;
;       - 16-bit variables:
HEXDATAH        .EQU    PGZERO_ST+10            ;Hexadecimal input
HEXDATAL        .EQU    PGZERO_ST+11
BINVALL         .EQU    PGZERO_ST+12            ;Binary Value for HEX2ASC
BINVALH         .EQU    PGZERO_ST+13
COMLO           .EQU    PGZERO_ST+14            ;User command address
COMHI           .EQU    PGZERO_ST+15
INDEXL          .EQU    PGZERO_ST+16            ;Index for address - multiple routines
INDEXH          .EQU    PGZERO_ST+17
TEMP1L          .EQU    PGZERO_ST+18            ;Index for word temp value used by Memdump
TEMP1H          .EQU    PGZERO_ST+19
TEMP2L          .EQU    PGZERO_ST+20            ;Index for Text entry
TEMP2H          .EQU    PGZERO_ST+21
PROMPTL         .EQU    PGZERO_ST+22            ;Prompt string address
PROMPTH         .EQU    PGZERO_ST+23
SRCL            .EQU    PGZERO_ST+24            ;Source address for memory operations
SRCH            .EQU    PGZERO_ST+25
TGTL            .EQU    PGZERO_ST+26            ;Target address for memory operations
TGTH            .EQU    PGZERO_ST+27
LENL            .EQU    PGZERO_ST+28            ;Length address for memory operations
LENH            .EQU    PGZERO_ST+29
;
;       - 8-bit variables and constants:
BUFIDX          .EQU    PGZERO_ST+30            ;Buffer index
BUFLEN          .EQU    PGZERO_ST+31            ;Buffer length
IDX             .EQU    PGZERO_ST+32            ;Temp Indexing
IDY             .EQU    PGZERO_ST+33            ;Temp Indexing
TEMP1           .EQU    PGZERO_ST+34            ;Temp - Code Conversion routines
TEMP2           .EQU    PGZERO_ST+35            ;Temp - Memory/EEPROM/SREC routines - Disassembler
TEMP3           .EQU    PGZERO_ST+36            ;Temp - EEPROM/SREC routines
CMDFLAG         .EQU    PGZERO_ST+37            ;Command Flag, bit specific, used by many routines
OPXMDM          .EQU    PGZERO_ST+38            ;Saved Opcode/Xmodem Flag variable
;
;       - Xmodem transfer variables
CRCHI           .EQU    PGZERO_ST+39            ;CRC hi byte  (two byte variable)
CRCLO           .EQU    PGZERO_ST+40            ;CRC lo byte - Operand in Disassembler
CRCCNT          .EQU    PGZERO_ST+41            ;CRC retry count - Operand in Disassembler
PTRL            .EQU    PGZERO_ST+42            ;Data pointer lo byte - Mnemonic in Disassembler
PTRH            .EQU    PGZERO_ST+43            ;Data pointer hi byte - Mnemonic in Disassembler
BLKNO           .EQU    PGZERO_ST+44            ;Block number
;
;        - Macro Loop Counter variables
LPCNTL          .EQU    PGZERO_ST+45            ;Loop Count low byte
LPCNTH          .EQU    PGZERO_ST+46            ;Loop Count high byte
;
;       - Spare Monitor byte for future use
SPARE_M0        .EQU    PGZERO_ST+47            ;Spare Monitor page zero byte
;
;       - BIOS variables, pointers, flags located at top of Page Zero
BIOS_PG0        .EQU    PGZERO_ST+48            ;Start of BIOS page 0 use ($D0-$FF, 48 bytes)
;
;       - BRK handler routine
PCL             .EQU    BIOS_PG0+00             ;Program Counter Low index
PCH             .EQU    BIOS_PG0+01             ;Program Counter High index
PREG            .EQU    BIOS_PG0+02             ;Temp Status Reg
SREG            .EQU    BIOS_PG0+03             ;Temp Stack ptr
YREG            .EQU    BIOS_PG0+04             ;Temp Y Reg
XREG            .EQU    BIOS_PG0+05             ;Temp X Reg
AREG            .EQU    BIOS_PG0+06             ;Temp A Reg
;
;       - 28L92 IRQ handler pointers and status
ICNT_A          .EQU    BIOS_PG0+07             ;Input buffer count
IHEAD_A         .EQU    BIOS_PG0+08             ;Input buffer head pointer
ITAIL_A         .EQU    BIOS_PG0+09             ;Input buffer tail pointer
OCNT_A          .EQU    BIOS_PG0+10             ;Output buffer count
OHEAD_A         .EQU    BIOS_PG0+11             ;Output buffer head pointer
OTAIL_A         .EQU    BIOS_PG0+12             ;Output buffer tail pointer
;
ICNT_B          .EQU    BIOS_PG0+13             ;Input buffer count
IHEAD_B         .EQU    BIOS_PG0+14             ;Input buffer head pointer
ITAIL_B         .EQU    BIOS_PG0+15             ;Input buffer tail pointer
OCNT_B          .EQU    BIOS_PG0+16             ;Output buffer count
OHEAD_B         .EQU    BIOS_PG0+17             ;Output buffer head pointer
OTAIL_B         .EQU    BIOS_PG0+18             ;Output buffer tail pointer
;
UART_SRT_A      .EQU    BIOS_PG0+19             ;SC28L92 Status Register Port A (not used)
UART_SRT_B      .EQU    BIOS_PG0+20             ;SC28L92 Status Register Port B (not used)
UART_IRT        .EQU    BIOS_PG0+21             ;SC28L92 Interrupt Status byte
;
;       - Real-Time Clock variables
; These are repurposed for adding a Realtime clock chip (DS1501/DS1511)
; The Ticks, Seconds, Minutes and Hours remain the same in function.
; The 16-bit Days variable is replaced however.
; - The DAY_DATE is a new variable. To minimize Page Zero usage, it has two functions
;       Bits 0-4 represent the days of the Month 1-31
;       Bits 5-7 represent the Day of the Week, 1-7 (Saturday=1)
; The Months are handled by the upper 4 bits of the MONTH_YEAR variable
; The Century is handled by a the Year (0-255) and the lower 4 bits of the MONTH_YEAR variable
;
TICKS           .EQU    BIOS_PG0+22             ;Number of timer countdowns = 1 second (100)
SECS            .EQU    BIOS_PG0+23             ;Seconds: 0-59
MINS            .EQU    BIOS_PG0+24             ;Minutes: 0-59
HOURS           .EQU    BIOS_PG0+25             ;Hours: 0-23
DAY_DATE        .EQU    BIOS_PG0+26             ;Day: (bits 5-7) Date: (bits 0-4)
MONTH_CENTURY   .EQU    BIOS_PG0+27             ;Month: (bits 4-7) Century: (bits 0-3)
YEAR            .EQU    BIOS_PG0+28             ;Century 0-255 plus 4 bits as noted above
RTC_TEMP        .EQU    BIOS_PG0+29             ;Temp work byte for updating shared variables
;
;       - Delay Timer variables
MSDELAY         .EQU    BIOS_PG0+30             ;Timer delay countdown byte (255 > 0)
SETMS           .EQU    BIOS_PG0+31             ;Set timeout for delay routines - BIOS use only
DELLO           .EQU    BIOS_PG0+32             ;Delay value BIOS use only
DELHI           .EQU    BIOS_PG0+33             ;Delay value BIOS use only
;
;       - Count variables for 10ms benchmark timing
MS10_CNT        .EQU    BIOS_PG0+34             ;10ms Count variable
SECL_CNT        .EQU    BIOS_PG0+35             ;Seconds Low byte count
SECH_CNT        .EQU    BIOS_PG0+36             ;Seconds High byte count
;
;       - Address and pointers for IDE Interface
LBA_ADDR_LOW    .EQU    BIOS_PG0+37             ;LBA Transfer Address low byte
LBA_ADDR_HIGH   .EQU    BIOS_PG0+38             ;LBA Transfer Address high byte
;
LBA_XFER_CNT    .EQU    BIOS_PG0+39             ;LBA Transfer Count 1-xx (check RAM space!)
LBA_LOW_BYTE    .EQU    BIOS_PG0+40             ;LBA Block number bits 0-7
LBA_HIGH_BYTE   .EQU    BIOS_PG0+41             ;LBA Block number bits 8-15
LBA_EXT_BYTE    .EQU    BIOS_PG0+42             ;LBA Block number bits 16-23
;
BIOS_XFERL      .EQU    BIOS_PG0+43             ;BIOS Move Routine low byte
BIOS_XFERH      .EQU    BIOS_PG0+44             ;BIOS Move Routine high byte
BIOS_XFERC      .EQU    BIOS_PG0+45             ;BIOS Block Count moved (needs to be set)
;
IDE_STATUS_RAM  .EQU    BIOS_PG0+46             ;IDE RAM-Based Status
;
;       - Timer/Counter Match flag for Delay/Benchmark
MATCH           .EQU    BIOS_PG0+47             ;Bit7 used for Delay, Bit6 used for Benchmark
                                                ;Bits 3,2,1 used for IDE Interrupt Handler
;
;       - Default for RTC tick count - number of IRQs for 1 second
DF_TICKS        .EQU    100                     ;Timer is 10 milliseconds (100 x 10ms = 1 second)
;
;**************************************************************************************************
IBUF_A          .EQU    $0200                   ;Console Input Buffer - 128 bytes
OBUF_A          .EQU    $0280                   ;Console Output Buffer - 128 bytes
;
IBUF_B          .EQU    $0400                   ;Alternate Input Buffer - 128 bytes
OBUF_B          .EQU    $0480                   ;Alternate Output Buffer - 128 bytes
;**************************************************************************************************
SOFTVEC         .EQU    $0300                   ;Start of soft vectors
;The Interrupt structure is vector based. During startup, Page $03 is loaded from ROM.
; The soft vectors are structured to allow inserting additional routines either before
; or after the ROM based routines. This allows flexibility and changing of routine priority.
;
;The main set of vectors occupy the first 16 bytes of Page $03. The ROM handler for
; NMI, BRK and IRQ jump to the first 3 vectors. The following 3 vectors are loaded with
; return addresses to the ROM handler for each. The following 2 vectors are the cold and
; warm entry points for the Monitor. After the basic initialization, the monitor is entered.
;
;The following vector set allows inserts, pre or post for NMI/BRK/IRQ. There a total of 8 inserts
; which occupy 16 bytes. They can be used as required.
; Currently, VECINSRT0 will be used if an IDE Controller is detected.
;
NMIVEC0         .EQU    SOFTVEC+00              ;NMI Vector Entry 0
BRKVEC0         .EQU    SOFTVEC+02              ;BRK Vector Entry 0
IRQVEC0         .EQU    SOFTVEC+04              ;IRQ Vector Entry 0
;
NMIRTVEC0       .EQU    SOFTVEC+06              ;NMI Vector Return 0
BRKRTVEC0       .EQU    SOFTVEC+08              ;BRK Vector Return 0
IRQRTVEC0       .EQU    SOFTVEC+10              ;IRQ Vector Return 0
;
CLDMNVEC0       .EQU    SOFTVEC+12              ;Monitor Cold Entry Vector 0
WRMMNVEC0       .EQU    SOFTVEC+14              ;Monitor Warm Entry Vector 0
;
VECINSRT0       .EQU    SOFTVEC+16              ;1st Vector Insert
VECINSRT1       .EQU    SOFTVEC+18              ;2nd Vector Insert
VECINSRT2       .EQU    SOFTVEC+20              ;3rd Vector Insert
VECINSRT3       .EQU    SOFTVEC+22              ;4th Vector Insert
VECINSRT4       .EQU    SOFTVEC+24              ;5th Vector Insert
VECINSRT5       .EQU    SOFTVEC+26              ;6th Vector Insert
VECINSRT6       .EQU    SOFTVEC+28              ;7th Vector Insert
VECINSRT7       .EQU    SOFTVEC+30              ;8th Vector Insert
;
;**************************************************************************************************
;
;Soft Config values below are loaded from ROM and are the default I/O setup Configuration data that
; the INIT_x routines use. As a result, you can write a routine to change the I/O Configuration
; data and use the standard ROM routines to initialize the I/O without restarting or changing ROM
; A Reset (HW or coded) will reinitialize the I/O with the ROM default I/O Configuration.
;
;There are a total of 32 Bytes Configuration data reserved starting at $0320,
; - 16 bytes reserved for the NXP SC28L92 DUART.
;
SOFTCFG         .EQU    SOFTVEC+32              ;Start of hardware Config parameters
;
LOAD_28L92      .EQU    SOFTCFG+00              ;SC28L92 Soft Config Data
;
; The configuration for the DUART consists of 14 parameters/commands stored in the following
; - memory locations. Note that these are sent in reverse, i.e., $32E thru $320.
;
;       $320    .DB     %00000011       $03     ;Enable OP0/1 for RTS control Port A/B
;       $321    .DB     %00001010       $A0     ;Disable Receiver/Disable Transmitter B
;       $322    .DB     %00001001       $09     ;Enable Receiver/Disable Transmitter A
;       $323    .DB     %00001111       $0F     ;Interrupt Mask Register setup
;       $324    .DB     %11100000       $E0     ;Aux Register setup for Counter/Timer
;       $325    .DB     %01001000       $48     ;Counter/Timer Upper Preset (18432 decimal)
;       $326    .DB     %00000000       $00     ;Counter/Timer Lower Preset
;       $327    .DB     %11001100       $CC     ;Baud Rate clock for B Rcv/Xmt - 115.2K
;       $328    .DB     %11001100       $CC     ;Baud Rate clock for A Rcv/Xmt - 115.2K
;       $329    .DB     %00110000       $30     ;Reset Transmitter B
;       $32A    .DB     %00100000       $20     ;Reset Receiver B
;       $32B    .DB     %00110000       $30     ;Reset Transmitter A
;       $32C    .DB     %00100000       $20     ;Reset Receiver A
;       $32D    .DB     %00000000       $00     ;Interrupt Mask Register setup (clear)
;       $32E    .DB     %11110000       $F0     ;Command Register A - Disable Power Down
;       $32F    .DB     %11111111       $FF     ;Spare Byte
;
; The MR registers of the DUART also have soft config data loaded here, but is separate from the
; - main register config data, as these are all accessed via a single I/O port (auto-indexed).
; - These are also sent in reverse order as above.
;
;       $330    .DB     %00010111       $17     ;Mode Register 2 data
;       $331    .DB     %11010011       $D3     ;Mode Register 1 Data
;       $332    .DB     %11111001       $F9     ;Mode Register 0 Data
;
;       $333    .DB     %00010111       $17     ;Mode Register 2 data
;       $334    .DB     %11010011       $D3     ;Mode Register 1 Data
;       $335    .DB     %11000001       $C1     ;Mode Register 0 Data
;
;       10 additional bytes all reserved as $FF fill up the remaining soft configuration data.
;
; The Microdrive is initialized and the total LBA count is save here during startup.
;  It is used by various utilities and allows drive capacity sensing without sending additional
;  commands to the Microdrive. A total of 4 bytes are used for a 32-bit LBA count. These are saved
;  at addresses $33C - $33F. Order is low-word/high-word. Each word is low-byte/high/byte.
;
LOAD_IDE        .EQU    SOFTCFG+28              ;IDE/CF-Card Soft Config Data
;
;       $33C    .DW                             ;Low order LBA count
;       $33E    .DW                             ;High order LBA count
;
;Search Buffer is 16 bytes in length. Used to hold search string for text or hex data
SRCHBUFF        .EQU    SOFTCFG+32              ;Located in Page $03 following Hardware Config data
;       $340                                    ;Start of search buffer (16 bytes)
;
;Xmodem/CRC Loader also provides Motorola S19 Record sense and load. Designed to handle the S19
; records from the WDC Assembler/Linker package. This requires a 44 byte buffer to parse each valid
; S1 record, located just before the 132 Byte Xmodem frame buffer. Total Buffer space for the
; Xmodem/CRC Loader is 176 bytes
;
;Valid S-record headers are "S1" and "S9". For S1, the maximum length is "$19" hex. The last S1
; record can be less. S9 record is always the last record with no data. WDC Linker also appends
; a CR/LF to the end of each record for a total of 44 bytes.
;
SRBUFF          .EQU    SOFTCFG+48              ;S-Record buffer, up to 44 bytes in length
;       $350                                    ;Start of S-Record buffer
;
;Xmodem frame buffer. The entire Xmodem frame is buffered here and then checked for proper header
; and frame number, CRC-16 on the data, then moved to user RAM.
RBUFF           .EQU    SOFTCFG+92              ;Xmodem temp 132 byte receive buffer
;       $37C                                    ;Start of Receive buffer for Xmodem
;
;Page $03 is completely allocated for Buffers, Config Data and Vector pointers.
; Some of the buffer space can be used as needed, provided any required Monitor functions are NOT
; being used concurrently.
;
;**************************************************************************************************
;
;Page $05 is used for the Realtime Clock NVRAM read and write routines
NVRAM_DATA      .EQU    $0500                   ;NVRAM Data Buffer address
;
;**************************************************************************************************
;
;Pages $06 - $07 are used for the IDE device Block Buffer (512 bytes)
LBA_BUFFER      .EQU    $0600                   ;Default IDE Block Buffer address
BOOT_BUFFER     .EQU    $0800                   ;Default IDE Boot Buffer address
;
;**************************************************************************************************
;XMODEM Control Character Constants
SOH             .EQU    $01                     ;Start of Block Header
EOT             .EQU    $04                     ;End of Text marker
ACK             .EQU    $06                     ;Good Block Acknowledge
NAK             .EQU    $15                     ;Bad Block Acknowledged
CAN             .EQU    $18                     ;Cancel character
;
;**************************************************************************************************
;RAM location used for the EEPROM Byte Write routine
; EEPROM is the address offset of the AT28BV256 in the hardware memory map and added to the
; EEPROM address locations required to unlock the AT28BV256 for insitu programming. For more
; information, refer to the AT28BV256 Datasheet.
;
BURN_BYTE       .EQU    $0070                   ;Page 0 RAM for EEPROM BYTE write routine
EEPROM          .EQU    $8000                   ;Offset to EEPROM in hardware
;**************************************************************************************************
;DOS/65 can be called from the Monitor via the Ctrl-B command.
;The start location is just added here for convenience, but should be changed if needed.
;
DOS_65          .EQU    $D630                   ;Default location to Boot DOS/65 (optional)
;**************************************************************************************************
IOPAGE          .EQU    $FE00                   ;I/O Page Base Start Address
;**************************************************************************************************
SC28L92_BASE    .EQU    IOPAGE+$80              ;Beginning of Console UART address
;
UART_MODEREG_A  .EQU    SC28L92_BASE+$00        ;MR0/MR1/MR2 Port A sequential (Read/Write)
UART_STATUS_A   .EQU    SC28L92_BASE+$01        ;UART Status Register Port A (READ)
UART_CLKSEL_A   .EQU    SC28L92_BASE+$01        ;UART Clock Select Port A (WRITE)
UART_RES_A      .EQU    SC28L92_BASE+$02        ;UART Reserved Port A (READ)
UART_COMMAND_A  .EQU    SC28L92_BASE+$02        ;UART Command Register Port A (WRITE)
UART_RECEIVE_A  .EQU    SC28L92_BASE+$03        ;UART Receive Register Port A (READ)
UART_TRANSMIT_A .EQU    SC28L92_BASE+$03        ;UART Transmit Register Port A (WRITE)
;
UART_PORT_CHG   .EQU    SC28L92_BASE+$04        ;UART Input Port Change Register (READ)
UART_AUXCR      .EQU    SC28L92_BASE+$04        ;UART Aux Command Register (WRITE)
UART_ISR        .EQU    SC28L92_BASE+$05        ;UART Interrupt Status Register (READ)
UART_IMR        .EQU    SC28L92_BASE+$05        ;UART Interrupt Mask Register (WRITE)
;
UART_CNTU       .EQU    SC28L92_BASE+$06        ;Counter/Timer Upper Register (READ)
UART_CNTUP      .EQU    SC28L92_BASE+$06        ;Counter/Timer Upper Preset Register (WRITE)
UART_CNTL       .EQU    SC28L92_BASE+$07        ;Counter/Timer Lower Register (READ)
UART_CNTLP      .EQU    SC28L92_BASE+$07        ;Counter/Timer Lower Preset Register (WRITE)
;
UART_MODEREG_B  .EQU    SC28L92_BASE+$08        ;MR0/MR1/MR2 Port B sequential Read/Write
UART_STATUS_B   .EQU    SC28L92_BASE+$09        ;UART Status Register Port B (READ)
UART_CLKSEL_B   .EQU    SC28L92_BASE+$09        ;UART Clock Select Port B (WRITE)
UART_RES_B      .EQU    SC28L92_BASE+$0A        ;UART Reserved Port B (READ)
UART_COMMAND_B  .EQU    SC28L92_BASE+$0A        ;UART Command Register Port B (WRITE)
UART_RECEIVE_B  .EQU    SC28L92_BASE+$0B        ;UART Receive Register Port B (READ)
UART_TRANSMIT_B .EQU    SC28L92_BASE+$0B        ;UART Transmit Register Port B (WRITE)
;
UART_MISC       .EQU    SC28L92_BASE+$0C        ;UART Miscellaneous Register Intel (Read/Write)
UART_INPUT_PORT .EQU    SC28L92_BASE+$0D        ;UART Input Port Register (READ)
UART_OUT_CFG    .EQU    SC28L92_BASE+$0D        ;UART Ouput Port Config Register (WRITE)
UART_START_CNT  .EQU    SC28L92_BASE+$0E        ;UART Start Counter Command (READ)
UART_SOPR_CMD   .EQU    SC28L92_BASE+$0E        ;UART Set Output Port Bits Register (WRITE)
UART_STOP_CNT   .EQU    SC28L92_BASE+$0F        ;UART Stop Counter Command (READ)
UART_ROPR_CMD   .EQU    SC28L92_BASE+$0F        ;UART Reset Output Port Bits Register (WRITE)
;
;Additional Hardware
; Adding BIOS definitions for Realtime Clock chip - DS1511
; uses the first 16 addresses for RTC registers and basic operation
; uses two addresses for extended RAM of 256 bytes
;
; upper addresses are used for a 16-bit IDE interface (below)
; NOTE: offset $11 and $12 are unused (reserved per the datasheet).
;
RTC_IDE_BASE    .EQU    IOPAGE+$60              ;Beginning of Realtime Clock address
;
RTC_SECONDS     .EQU    RTC_IDE_BASE+$00        ;Seconds in BCD 00-59
RTC_MINUTES     .EQU    RTC_IDE_BASE+$01        ;Minutes in BCD 00-59
RTC_HOURS       .EQU    RTC_IDE_BASE+$02        ;Hours in BCD 00-23
RTC_DAY         .EQU    RTC_IDE_BASE+$03        ;Day in BCD 1-7
RTC_DATE        .EQU    RTC_IDE_BASE+$04        ;Date in BCD 1-31
RTC_MONTH       .EQU    RTC_IDE_BASE+$05        ;Month in BCD 1-12
RTC_YEAR        .EQU    RTC_IDE_BASE+$06        ;Year in BCD 00-99
RTC_CENTURY     .EQU    RTC_IDE_BASE+$07        ;Century in BCD 00-39
;
RTC_ALARM_SEC   .EQU    RTC_IDE_BASE+$08        ;Alarm Seconds in BCD 00-59
RTC_ALARM_MIN   .EQU    RTC_IDE_BASE+$09        ;Alarm Minutes in BCD 00-59
RTC_ALARM_HRS   .EQU    RTC_IDE_BASE+$0A        ;Alarm Hours in BCD 00-23
RTC_ALARM_DYDT  .EQU    RTC_IDE_BASE+$0B        ;Alarm Day/Date in BCD 0-7 1-31
RTC_WTCHDOG_01  .EQU    RTC_IDE_BASE+$0C        ;Watchdog 0.1 / 0.01 Seconds in BCD 00-99
RTC_WTCHDOG_10  .EQU    RTC_IDE_BASE+$0D        ;Watchdog 10 / 1 Seconds in BCD 00-99
;
RTC_CONTROL_A   .EQU    RTC_IDE_BASE+$0E        ;Control A
RTC_CONTROL_B   .EQU    RTC_IDE_BASE+$0F        ;Control B
RTC_RAM_ADDR    .EQU    RTC_IDE_BASE+$10        ;Extended RAM address
RTC_RAM_DATA    .EQU    RTC_IDE_BASE+$13        ;Extended RAM data
;
; Adding BIOS definitions for 16-bit IDE interface
; uses two addresses for Upper Byte Latch read / write
; uses eight addresses for Command Block Registers
; uses two addresses for Control Block Registers
;
IDE_16_READ     .EQU    RTC_IDE_BASE+$14        ;Upper byte Read address
IDE_16_WRITE    .EQU    RTC_IDE_BASE+$15        ;Upper byte Write address
;
; Adding BIOS definitions for IDE Controller (HARD DISK, Flash Module, etc.)
; Hardware Adapter provides a 16-bit IDE Port per:
;  Seagate ATA Interface Reference Manual 36111-001, Rev. C (21st May 1993)
;
; Control Block Registers
IDE_ALT_STATUS  .EQU    RTC_IDE_BASE+$16        ;Alternate Status Register (READ)
IDE_DEV_CTRL    .EQU    RTC_IDE_BASE+$16        ;Device Control Register (WRITE)
IDE_DRV_ADDR    .EQU    RTC_IDE_BASE+$17        ;Drive Address Register (READ)
;
; Command Block Registers
IDE_DATA        .EQU    RTC_IDE_BASE+$18        ;Data Register (R/W)
IDE_ERROR       .EQU    RTC_IDE_BASE+$19        ;Error Register (READ)
IDE_FEATURE     .EQU    RTC_IDE_BASE+$19        ;Feature Register (WRITE)
IDE_SCT_CNT     .EQU    RTC_IDE_BASE+$1A        ;Sector Count Register
IDE_SCT_NUM     .EQU    RTC_IDE_BASE+$1B        ;Sector Number Register
IDE_CYL_LOW     .EQU    RTC_IDE_BASE+$1C        ;Cylinder Low Register
IDE_CYL_HIGH    .EQU    RTC_IDE_BASE+$1D        ;Cylinder High Register
IDE_DRV_HEAD    .EQU    RTC_IDE_BASE+$1E        ;Drive/Head Register
IDE_STATUS      .EQU    RTC_IDE_BASE+$1F        ;Status Register (READ)
IDE_COMMAND     .EQU    RTC_IDE_BASE+$1F        ;Command Register (WRITE)
;
;**************************************************************************************************
;       BIOS JUMP Table starts at $FF00
;       - BIOS calls are listed below - total of 32
;       - Reserved calls are for future hardware support
;**************************************************************************************************
;
B_IDE_RESET             .EQU    $FF00   ;Call 00
B_IDE_GET_STAT          .EQU    $FF03   ;Call 01
B_IDE_IDENTIFY          .EQU    $FF06   ;Call 02
B_IDE_READ_LBA          .EQU    $FF09   ;Call 03
B_IDE_WRITE_LBA         .EQU    $FF0C   ;Call 04
B_IDE_VERFY_LBA         .EQU    $FF0F   ;Call 05
B_IDE_SET_LBA           .EQU    $FF12   ;Call 06
B_IDE_SET_ADDR          .EQU    $FF15   ;Call 07
B_IDE_EN_CACHE          .EQU    $FF18   ;Call 08
B_IDE_DIS_CACHE         .EQU    $FF1B   ;Call 09
;
B_RTC_NVRD              .EQU    $FF1E   ;Call 10
B_RTC_NVWR              .EQU    $FF21   ;Call 11
B_RTC_INIT              .EQU    $FF24   ;Call 12
;
B_CHRIN2                .EQU    $FF27   ;Call 13
B_CHROUT2               .EQU    $FF2A   ;Call 14
;
B_CNT_INIT              .EQU    $FF2D   ;Call 15
B_CNT_STRT              .EQU    $FF30   ;Call 16
B_CNT_STOP              .EQU    $FF33   ;Call 17
;
B_CHRIN_NW              .EQU    $FF36   ;Call 18
B_CHRIN                 .EQU    $FF39   ;Call 19
B_CHROUT                .EQU    $FF3C   ;Call 20
;
B_SET_DLY               .EQU    $FF3F   ;Call 21
B_EXE_MSDLY             .EQU    $FF42   ;Call 22
B_EXE_LGDLY             .EQU    $FF45   ;Call 23
;
B_RESERVE               .EQU    $FF48   ;Call 24
;
B_INIT_VEC              .EQU    $FF4B   ;Call 25
B_INIT_CFG              .EQU    $FF4E   ;Call 26
B_INIT_28L92            .EQU    $FF51   ;Call 27
B_RESET_28L92           .EQU    $FF54   ;Call 28
;
B_WRMMNVEC0             .EQU    $FF57   ;Call 29
B_CLDMNVEC0             .EQU    $FF5A   ;Call 30
B_COLDSTRT              .EQU    $FF5D   ;Call 31
;
BIOS_MSG                .EQU    $FFD0   ;BIOS Startup Message is hard-coded here
;
;**************************************************************************************************
;The following 32 functions are provided by the Monitor via the JMP Table below
; $E006 - $E00C are Reserved for future expansion
;
; $E000 COLD_Monitor    ;Monitor Cold start entry
; $E003 WARM_Monitor    ;Monitor Warm start entry
;
; $E006 RESERVE2        ;Reserved function call
; $E009 RESERVE3        ;Reserved function call
; $E00C RESERVE4        ;Reserved function call
;
; $E00F BSOUT           ;Send a Backspace to console
; $E012 XMDM_SAVE       ;Xmodem Save - entry point for external use
; $E015 XMDM_LOAD       ;Xmodem Load - entry point for external use
; $E018 BENCH           ;Start Benchmark Timer (resets count)
; $E01B QUITB           ;Quits Benchmark Timer (displays elapsed time)
; $E01E DATETIME        ;Displays SW-RTC Date/Time
; $E021 PRSTAT          ;Displays Processor Status
; $E024 DIS_LINE        ;Disassembles a line of code (Mnemonic/Operand)
; $E027 INCINDEX        ;Increments 16-bit Index pointer
; $E02A DECINDEX        ;Decrements 16-bit Index pointer
; $E02D RDLINE          ;Read a Line of Data from console
; $E030 RDCHAR          ;Read a Character from console
; $E033 HEXIN2          ;Get two Hex digits from console entry (1-byte)
; $E036 HEXIN4          ;Get four Hex digits from console entry (2-bytes)
; $E039 HEX2ASC         ;Converts Hex into ASCII (1-byte to 2-ASCII digits)
; $E03C BIN2ASC         ;Converts Binary to ASCII
; $E03F ASC2BIN         ;Converts two ASCII digits to one byte
; $E042 BEEP            ;Send a Bell character to console
; $E045 DOLLAR          ;Send a $ character to console
; $E048 CROUT           ;Send a CR/LF sequence to console
; $E04B SPC             ;Send a Space character to console
; $E04E PRBYTE          ;Print a byte as two ASCII digits to console
; $E051 PRWORD          ;Print a word as four ASCII digits to console
; $E054 PRASC           ;Print ASCII character to console
; $E057 PROMPT          ;Print a Message number to console
; $E05A PROMPTR         ;Print a String to console (via 16-bit address)
; $E05D CONTINUE        ;Prompt to continue, else pull return address from stack and return
;
;**************************************************************************************************
;       Monitor JUMP table starts at $E000
;       - Monitor calls are listed below - total of 32
;       - Reserved calls are for future functions
;**************************************************************************************************
;
M_COLD_MON      .EQU    $E000           ;Call 00        Monitor Cold Start
M_WARM_MON      .EQU    $E003           ;Call 01        Monitor Warm Start
;
M_RESERVE2      .EQU    $E006           ;Call 02        Reserved
;
M_MOVE_RAM      .EQU    $E009           ;Call 03        Move Memory
M_FILL_LP       .EQU    $E00C           ;Call 04        Fill Memory
M_BSOUT         .EQU    $E00F           ;Call 05        Send Backspace
M_XMDM_SAVE     .EQU    $E012           ;Call 06        Xmodem Save Entry
M_XMDM_LOAD     .EQU    $E015           ;Call 07        Xmodem Load Entry
M_BENCH         .EQU    $E018           ;Call 08        Benchmark Start
M_QUITB         .EQU    $E01B           ;Call 09        Benchmark Stop/End
M_TIME          .EQU    $E01E           ;Call 10        System Date/Time
M_PRSTAT1       .EQU    $E021           ;Call 11        CPU Status Display
M_DIS_LINE      .EQU    $E024           ;Call 12        Disassemble Line of Code
M_INCINDEX      .EQU    $E027           ;Call 13        Increment Index by 1
M_DECINDEX      .EQU    $E02A           ;Call 14        Decrement Index by 1
M_RDLINE        .EQU    $E02D           ;Call 15        Read Line from Terminal
M_RDCHAR        .EQU    $E030           ;Call 16        Read Character from Terminal
M_HEXIN2        .EQU    $E033           ;Call 17        Hex input 2 characters
M_HEXIN4        .EQU    $E036           ;Call 18        Hex input 4 characters
M_HEX2ASC       .EQU    $E039           ;Call 19        Convert Hex to ASCII
M_BIN2ASC       .EQU    $E03C           ;Call 20        Convert Binary to ASCII
M_ASC2BIN       .EQU    $E03F           ;Call 21        Convert ASCII to Binary
M_BEEP          .EQU    $E042           ;Call 22        Send BEEP to Terminal
M_DOLLAR        .EQU    $E045           ;Call 23        Send $ to Terminal
M_CROUT         .EQU    $E048           ;Call 24        Send C/R to Terminal
M_SPC           .EQU    $E04B           ;Call 25        Send ASCII Space to Terminal
M_PRBYTE        .EQU    $E04E           ;Call 26        Print Byte to Terminal
M_PRWORD        .EQU    $E051           ;Call 27        Print Word to Terminal
M_PRASC         .EQU    $E054           ;Call 28        Print ASCII to Terminal
M_PROMPT        .EQU    $E057           ;Call 29        Send Message by number to Terminal
M_PROMPTR       .EQU    $E05A           ;Call 30        Send Message by address to Terminal
M_CONTINUE      .EQU    $E05D           ;Call 31        Y/N Prompt to Continue Command
;
;**************************************************************************************************
;
;       User program code can start here. Default to $0800, can be changed as required
;
;**************************************************************************************************
;
        LIST                                    ;Turn listing on for the good stuff
;
; A few extra page zero variables for our use:
START_ADDR      .EQU    $00     ;Use the first two bytes for start address
LENGTH_ADDR     .EQU    $02     ;Use the next pair for the length of test
FILL_BYTE       .EQU    $04     ;Fill Byte for Fill/Compare routine
ITERATIONS      .EQU    $05     ;Number of times to test something, memory??
B_DATA          .EQU    $06     ;Failing pattern
;
        .ORG    $0400   ;Start of User RAM for programs
;
; Classic memory test program. The usual test patterns first:
;       $FF
;       $AA
;       $55
;       $00
; Followed by the "walking bit" test trying to unsettle memory under fast read/modify/write ops.
;
; First prompt the user for a start address, length of test and the number of iterations.
; shows errors as failing address and pattern that failed.
;
RAM_TEST
;
; Give us an intro message..
        LDA     #<OPEN          ;Get message address
        LDY     #>OPEN          ;
        JSR     M_PROMPTR       ;Use Monitor routine to send it
;
RINSE_REPEAT
; Prompt the user for a starting address:
        LDA     #<ENTRY         ;Get message address
        LDY     #>ENTRY         ;
        JSR     M_PROMPTR       ;Use Monitor routine to send it
;
;Get the starting address in hex:
; address returned in A/Y: low/high
        JSR     M_HEXIN4        ;Use Monitor routine to get a 16-bit address
        STA     START_ADDR      ;Save it for the test
        STY     START_ADDR+1    ;Save the high byte too
;
;Get the length of the test in hex:
; Prompt the user for an starting address:
        LDA     #<ENTRY1        ;Get message address
        LDY     #>ENTRY1        ;
        JSR     M_PROMPTR       ;Use Monitor routine to send it
;
;Get the length in hex:
; address returned in A/Y: low/high
        JSR     M_HEXIN4        ;Use Monitor routine to get a 16-bit address
        STA     LENGTH_ADDR     ;Save it for the test
        STY     LENGTH_ADDR+1   ;Save the high byte too
;
;Get the number of iterations in hex:
        LDA     #<ENTRY2        ;Get message address
        LDY     #>ENTRY2        ;
        JSR     M_PROMPTR       ;Use Monitor routine to send it
;
; side note: $00 entered will loop 256 times, buy that?
        JSR     M_HEXIN2        ;Get a single 8-bit value
        STA     ITERATIONS      ;Save it
;
; Send in progress message
        LDA     #<TESTING       ;Get message address
        LDY     #>TESTING       ;
        JSR     M_PROMPTR       ;Use Monitor routine to send it
;
RETEST
;Check for entered length zero, bad user,
; - do not pass go, do not 200 bytes 
        LDA     LENGTH_ADDR
        ORA     LENGTH_ADDR+1
        BEQ     EXIT_OK         ;NO ERRORS IF LENGTH IS ZERO
;
;First test, fill of $FF and compare it
        LDA     #$FF            ;Get the test pattern
        STA     FILL_BYTE       ;Save it to fill byte location
        JSR     FILL_COMPARE    ;Call our test routine
        BCS     FIL_CMP_ERR     ;If carry set, we tripped
;Second test, fill of $AA and compare it
        LDA     #$AA            ;Get the test pattern
        STA     FILL_BYTE       ;Save it to fill byte location
        JSR     FILL_COMPARE    ;Call our test routine
        BCS     FIL_CMP_ERR     ;If carry set, we tripped
;
;Third test, fill of $55 and compare it
        LDA     #$55            ;Get the test pattern
        STA     FILL_BYTE       ;Save it to fill byte location
        JSR     FILL_COMPARE    ;Call our test routine
        BCS     FIL_CMP_ERR     ;If carry set, we tripped
;
;Forth test, fill of $00 and compare it
        STZ     FILL_BYTE       ;Zero the fill byte location
        JSR     FILL_COMPARE    ;Call our test routine
        BCS     FIL_CMP_ERR     ;If carry set, we tripped
;
;I'm bored now... let's try something different..
; - I know, how about taking your bits for a walk?
;
;Fifth test, walk your bits left to right...
; - now this is more like it.
        JSR INIT_TEMPS          ;Initialize the Pointers
;
WALK_LP
        LDA     LENL            ;Get length low byte
        ORA     LENH            ;OR in length high byte
        BEQ     EXIT_OK         ;If no more, we're done
;
        LDA     #$80            ;Set high bit only
BIT_LP
        STA     (TGTL)          ;Store the bit pattern
        CMP     (TGTL)          ;Compare it back
        BNE     EXIT_ERR        ;Branch if we lost it
        LSR     A               ;Shift it to the right
        BNE     BIT_LP          ;Loop back if we still have a bit on
        STA     (TGTL)          ;Store a zero, clear RAM
        JSR     UPD_TL          ;Go update the pointers
        BRA     WALK_LP         ;Loop back until done
;
EXIT_OK
;
; Test completed... now see if we need to run it again
; - remember the definition of insanity before repeating, just sayin'
;
        DEC     ITERATIONS      ;Decrement the count
        BEQ     WE_DONE         ;We're done if zero
        BRA     RETEST          ;Or loop back
;
WE_DONE
;Print completed message
;
        LDA     #<COMPLETE      ;Get message address
        LDY     #>COMPLETE      ;
        JSR     M_PROMPTR       ;Use Monitor routine to send
;
        JSR     M_CONTINUE      ;Use Monitor routine for this
;
        JSR     M_CROUT         ;Send a C/R
        JMP     RINSE_REPEAT    ;Back to the bar for another round
;
EXIT_ERR
        LDA     (TGTL)          ;Get the failed data byte
        STA     B_DATA          ;Save it as collateral damage and blackmail potential
;
;**************************************************************************************************
;
FIL_CMP_ERR
;We're here because we forgot what we were told...
; - can't really remember what that was, but...
; - We need to show some stuff... so someone can fix us.
;
; First, show a message that life is not that good afterall
;
        LDA     #<BADMEMORY     ;Get message address
        LDY     #>BADMEMORY     ;
        JSR     M_PROMPTR       ;Use Monitor routine to send
;
; Show the actual bad address
        LDA     TGTL            ;Get low byte fail address
        LDY     TGTH            ;Get high byte fail address
        JSR     M_PRWORD        ;Use Monitor routine to send it
;
; Now show what I was suppose to remember
        LDA     #<PATTERN       ;Get Pattern message
        LDY     #>PATTERN       ;
        JSR     M_PROMPTR       ;Use Monitor routine to send
        LDA     FILL_BYTE       ;Get actual data pattern
        JSR     M_PRBYTE        ;Use Monitor routine to send it
;
; Now show what I thought I was told to remember
        LDA     #<RECALL        ;Get Pattern message
        LDY     #>RECALL        ;
        JSR     M_PROMPTR       ;Use Monitor routine to send
        LDA     B_DATA          ;Get bad data pattern
        JSR     M_PRBYTE        ;Use Monitor routine to send it
;
        JMP     M_CROUT         ;Send a C/R and return
;
;******************************************************************************
;Fill and Compare routine
;
; We take what's in the Fill_Byte and use that as a fill pattern.
; After filling the range of memory, we go back to where we never left,
; and compare it to what we just filled it with. If it matches we're good.
; If it doesn't, we lost our mind.... and need to figure out where and how.
;
; On exit, we set the carry flag for an error, or clear it for no error.
; - how original is that?
;
;******************************************************************************
FILL_COMPARE
        JSR INIT_TEMPS          ;Init the temp areas for filling
;
;Memory fill routine: parameters entered above,
; then a JSR to here, FILL_BYTE contains fill byte value (still)
FILL_LP
        LDA     LENL            ;Get length low byte
        ORA     LENH            ;OR in length high byte
        BEQ     DO_COMPARE      ;If done, go do the compare
        LDA     FILL_BYTE       ;Get fill byte
        STA     (TGTL)          ;Store in target location
        JSR     UPD_TL          ;Update Target/Length pointers
        BRA     FILL_LP         ;Loop back until done
;
;Memory compare routine: uses the same parameters again
; use the init_temps routine, then drop into this.
;
DO_COMPARE
        JSR INIT_TEMPS          ;Init the temp areas for comparing
COMP_LP
        LDA     LENL            ;Get length low byte
        ORA     LENH            ;OR in length high byte
        BEQ     FC_DONE         ;If done, go exit
        LDA     FILL_BYTE       ;Get fill byte
        CMP     (TGTL)          ;Compare to target location
        BNE     COMP_ERR        ;Opps, I lost some bits
        JSR     UPD_TL          ;Update Target/Length pointers
        BRA     COMP_LP         ;Loop back until done
;
FC_DONE
        CLC                     ;Clear carry for no error
        RTS                     ;Return to caller
;
COMP_ERR
;Compare Error... we failed miserably... or succeeded immensely, depending on your point of view
; - we do have an actual memory error, as far as I can recall.  To net things out:
; - FILL_BYTE is what we were told to remember (by someone)
; - What's at the (TGTL) memory address is what we remembered, which is apparently wrong
; - The actual TGTL/TGTH 16-bit pointer is where we fell on our sword and lost our bits
; - I hate when this happens!
;
        LDA     (TGTL)          ;Get the failed data byte
        STA     B_DATA          ;Save it as collateral damage and blackmail potential
        SEC                     ;Set carry, before we forget that too...
        RTS                     ;Return to the caller
;
;**************************************************************************************************
;
; Routines that take entered parameter data and setup the pointers to run the tests.
; Init_Temps routine:
; - This transfers the pointers for starting address and length of the memory range for testing.
;
INIT_TEMPS
;
        LDA     START_ADDR      ;Get Start address low byte
        LDY     START_ADDR+1    ;Get Start address high byte
        STA     TGTL            ;Save it to working target low byte
        STY     TGTH            ;Save it to working target high byte
;
        LDA     LENGTH_ADDR     ;Get Start address low byte
        LDY     LENGTH_ADDR+1   ;Get Start address high byte
        STA     LENL            ;Save it to working target low byte
        STY     LENH            ;Save it to working target high byte
        RTS                     ;Return to caller
;
;Routines to update pointers for memory operations. UPD_STL subroutine: Increments
; the Target pointers, then drops into DECLEN to descrement the Length pointers.
; all memory test routines use these routines.
;
;UPT_TL routine: increments the 16-bit variable TGTL/TGTH
UPD_TL
        INC     TGTL            ;Increment target low byte
        BNE     DECLEN          ;Check for rollover
        INC     TGTH            ;Increment target high byte
;
;DECLEN subroutine: decrement 16-bit variable LENL/LENH
DECLEN
        LDA     LENL            ;Get length low byte
        BNE     SKP_LENH        ;Test for LENL = zero
        DEC     LENH            ;Else decrement length high byte
SKP_LENH
        DEC     LENL            ;Decrement length low byte
        RTS                     ;Return to caller
;
;**************************************************************************************************
;
MESSAGES
;
OPEN
        .DB     $0D,$0A,$0A
        .DB     "C02 RAM Test - 'cause you never know when your memory goes!"
        .DB     $0D,$0A,$0A
        .DB     "All data entries are in Hexadecimal! (yea, 0 thru F)"
        .DB     $0D,$0A
        .DB     "Note: this memory test occupies $0400 - $07A1!"
        .DB     $0D,$0A
        .DB     "Don't test here, or you won't see me again!"
        .DB     $0D,$0A
        .DB     $00
;
ENTRY
        .DB     $0D,$0A
        .DB     "Enter the starting address: "
        .DB     $00
;
ENTRY1
        .DB     $0D,$0A
        .DB     "Enter the length of memory tested: "
        .DB     $00
;
ENTRY2
        .DB     $0D,$0A
        .DB     "Enter the number of interations: "
        .DB     $00
;
TESTING
        .DB     $0D,$0A
        .DB     "Relax, wait while I hammer your memory to bits... is it Happy Hour yet?"
        .DB     $0D,$0A
        .DB     $00
;
COMPLETE
        .DB     $0D,$0A
        .DB     "Memory test completed without errors, you should remember everything!"
        .DB     $0D,$0A,$0A
        .DB     "Time for another round, or go home? "
        .DB     $00
;
BADMEMORY
        .DB     $0D,$0A
        .DB     "Uh-oh! I lost my mind, where am I?"
        .DB     $0D,$0A
        .DB     " Ah, here's where I fell and banged my head: $"
        .DB     $00
;
PATTERN
        .DB     $0D,$0A
        .DB     "This is what they said they said: $"
        .DB     $00
;
RECALL
        .DB     $0D,$0A
        .DB     "This is what I think they said: $"
        .DB     $00
;
;**************************************************************************************************
;
        .END



